###############################################################################################################
#
#  Revision #          :  2.3
#  Last Updated        :  Oct 28, 2009
#  Author/Contributor  :  David Lin, dalin@cisco.com
#
#  Description         :  This EEM Tcl script will generate an alarm when a VPN failure occurs.
#                         A VPN failure is defined as failure to reach a set of remote peer's L3 tunnel interface(s)
#                         that are configured using GRE+IPSec or DMVPN.  
#
#  Requirements        :  -EEM env variables-    
#                         event manager environment vpn_email_server <your-mailserver-ipaddress or dns-name>
#                         event manager envrionment vpn_email_from <your-email-from-address>
#                         event manager environment vpn_email_to <your-email-to-address>
#
#                         Example: event manager environment vpn_email_server 10.10.10.10
#                                  event manager environment vpn_email_from router-123@cisco.com
#                                  event manager environment vpn_email_to noc@cisco.com
#
#                         -IOS CLI commands-
#                          crypto logging session 
#
#                         -Topology-                         
#                         Assumes an alternate path to reach the mail server will be available when tunnel is down. 
#
#                         -EEM trigger-
#                         Crypto syslog message that takes the form:
#                         %CRYPTO-5-SESSION_STATUS: Crypto tunnel is DOWN
#
#                         Note: The 'crypto logging session' CLI cmd is required to enable the crypto syslog.
#
#                         -EEM action-
#                         Sends an email message along with diagnostic information to help troubleshoot
#                         by operations staff.
#
#
#  Cisco Products tested :      Cisco 1841, Cisco2821
#
#  Cisco IOS Version tested :   12.4(9)T4
#
#
#  Revisioning           : 2.3 adds show clock information for timestamping
#
#                          2.2 (1) creates status file named vpn_failure_status.txt
#                              (2) moves event register after this comment section  
#
###############################################################################################################

::cisco::eem::event_register_syslog occurs 1  pattern .*%CRYPTO-5-SESSION_STATUS:.*Crypto.*tunnel.*is.*DOWN.* maxrun 60 priority low nice 1


#
# Namespace imports
#
namespace import ::cisco::eem::*
namespace import ::cisco::lib::*


#-------------------   hostname      -------------------
set routername [info hostname]


#
#-------  Fetch the syslog message that caused the event to trigger --------
#
array set arr_einfo [event_reqinfo]
if {$_cerrno != 0} {
    set result [format "component=%s; subsys err=%s; posix err=%s;\n%s" \
        $_cerr_sub_num $_cerr_sub_err $_cerr_posix_err $_cerr_str]
    error $result 
}

set syslog_msg $arr_einfo(msg)
#-----------------  End of fetch the syslog message that caused the event to trigger ----------


#-------  Extract Peer ID from syslog message and use for 'show ip route' command later -------
# Example syslog output : %CRYPTO-5-SESSION_STATUS: Crypto tunnel is DOWN.  Peer 10.10.10.1:500       Id: 10.10.10.1
# We only care about the value after the word "Peer:" which is 10.10.10.1 in the above example.  Keep this value
# stored in variable 'remote_peer'

regexp {.*%CRYPTO-5-SESSION_STATUS: Crypto tunnel is DOWN.*Peer ([0-9].*\.[0-9].*\.[0-9].*\.[0-9].*):[0-9].*} $syslog_msg match remote_peer

#puts "Peer ID is $remote_peer"
#action_syslog msg "Peer ID is $remote_peer"   


#-------------------   " cli open"   -------------------
if [catch {cli_open} result] {
  error $result $errorInfo
} else {
  array set cli $result
}

#--------------- end of  "cli open"   -------------------


#
#----------------------- "show commands" ----------------
#
  if [catch {cli_exec $cli(fd) "enable"} result] {
    error $result $errorInfo
  }

  if [catch {cli_exec $cli(fd) "show clock detail | inc UTC"} result] {
      error $result $errorInfo
  }
  set show_clock $result

  if [catch {cli_exec $cli(fd) "show ip int brief | inc up                    down"} result] {
      error $result $errorInfo
  }
  
  set show_ip_interface_brief_up_down $result
  
  if [catch {cli_exec $cli(fd) "show ip route $remote_peer"} result] {
    error $result $errorInfo
  }
  set show_ip_route $result

  if [catch {cli_exec $cli(fd) "show crypto isakmp sa"} result] {
    error $result $errorInfo
  }
  set show_crypto_isakmp_sa $result
  
  if [catch {cli_exec $cli(fd) "show crypto engine connection active"} result] {
    error $result $errorInfo
  }
  set show_crypto_engine_connection_active $result
  
  if [catch {cli_exec $cli(fd) "show ip nhrp detail"} result] {
    error $result $errorInfo
  }
  set show_ip_nhrp_detail $result
  
  if [catch {cli_exec $cli(fd) "show log"} result] {
    error $result $errorInfo
  }
  set show_log $result


#---------------------- end of show commands ------------


#
#----------------------- "enable" ----------------------
#
  if [catch {cli_exec $cli(fd) "enable"} result] {
    error $result $errorInfo
  }


#
#----------------------- format diagnostic file ----------------------
#


# fetch environment vars
if [catch {cli_exec $cli(fd) "sh ev man env vpn_failure_email"} result] {
    error $result $errorInfo
  }
set sh_ev_man_env_email $result

if [catch {cli_exec $cli(fd) "sh ev man env vpn_failure_diag"} result] {
    error $result $errorInfo
  }
set sh_ev_man_env_diag $result


# format to be used if email notification is set; sets the header properly
if { [lindex $sh_ev_man_env_email 0] == "y"} {
  set body [format "Mailservername: %s" "$vpn_email_server"]
  set body [format "%s\nFrom: %s" "$body" "$vpn_email_from"]
  set body [format "%s\nTo: %s" "$body" "$vpn_email_to"]
  set _email_cc ""
  set body [format "%s\nCc: %s" "$body" ""]
  set body [format "%s\nSubject: %s\n" "$body" "VPN Failure Detected: Router $routername Crypto tunnel is DOWN. Peer $remote_peer"]
} 

# format to be used if diag is enabled but email notification is disabled 
if { [lindex $sh_ev_man_env_diag 0] == "y" && [lindex $sh_ev_man_env_email 0] == "n" } {
  set body [format "%s\n%s" " " " "]
} 

# format to be used if both diag && email notification are disabled 
if { [lindex $sh_ev_man_env_diag 0] == "n" && [lindex $sh_ev_man_env_email 0] == "n" } {
  set body [format "%s\n%s" " " " "]
  action_syslog msg "Neither file creation nor email notification have been enabled for this script!"
}  

# general formating
  set body [format "%s\n%s" "$body" "Report Summary:"]
  set body [format "%s\n%s" "$body" "   - hostname"]
  set body [format "%s\n%s" "$body" "   - show clock detail"]
  set body [format "%s\n%s" "$body" "   - syslog message"]
  set body [format "%s\n%s" "$body" "   - summary of interface(s) in an up/down state"]
  set body [format "%s\n%s" "$body" "   - show ip route $remote_peer"]
  set body [format "%s\n%s" "$body" "   - show crypto isakmp sa"]
  set body [format "%s\n%s" "$body" "   - show crypto engine connection active"]
  set body [format "%s\n%s" "$body" "   - show ip nhrp detail (DMVPN only)"]
  set body [format "%s\n%s" "$body" "   - show log"]
  set body [format "%s\n\n%s" "$body" "---------- hostname ----------"]
  set body [format "%s\n%s" "$body" "$routername"]
  set body [format "%s\n\n%s" "$body" "---------- show clock detail ----------"]
  set body [format "%s\n%s" "$body" "$show_clock"]
  set body [format "%s\n\n%s" "$body" "---------- syslog message ----------"]
  set body [format "%s\n%s" "$body" "$syslog_msg"]
  set body [format "%s\n\n%s" "$body" "---------- summary of interface(s) in an up/down state ----------"]
  set body [format "%s\n\n%s" "$body" "$show_ip_interface_brief_up_down"]
  set body [format "%s\n\n%s" "$body" "---------- show ip route $remote_peer ----------"]
  set body [format "%s\n\n%s" "$body" "$show_ip_route"]
  set body [format "%s\n\n%s" "$body" "---------- show crypto isakmp sa ----------"]
  set body [format "%s\n\n%s" "$body" "$show_crypto_isakmp_sa"]
  set body [format "%s\n\n%s" "$body" "---------- show crypto engine connection active ----------"]
  set body [format "%s\n\n%s" "$body" "$show_crypto_engine_connection_active"]
  set body [format "%s\n\n%s" "$body" "---------- show ip nhrp detail (DMVPN only) ----------"]
  set body [format "%s\n\n%s" "$body" "$show_ip_nhrp_detail"]
  set body [format "%s\n\n%s" "$body" "---------- show log ----------"]
  set body [format "%s\n\n%s" "$body" "$show_log"]



#---------------------- write to file ------------------------  

if [catch {cli_exec $cli(fd) "show event manager env vpn_failure_diag"} result] {
    error $result $errorInfo
  }
set show_event_manager_env_diag $result

if [catch {cli_exec $cli(fd) "show event manager dir user policy"} result] {
    error $result $errorInfo
  }
set show_event_manager_dir $result

if {[lindex $show_event_manager_env_diag 0] == "y"} {
 
   set fd [open "[lindex $show_event_manager_dir 0]/vpn_failure_diag.txt" "w+"]
           puts "Writing diagnostics to [lindex $show_event_manager_dir 0]/vpn_failure_diag.txt"
           puts $fd $body
   close $fd
   

action_syslog msg "Diagnostics successfully written to [lindex $show_event_manager_dir 0]/vpn_failure_diag.txt"
}


#----------------------- send email ----------------------

if {[lindex $sh_ev_man_env_email 0] == "y"} {

  action_syslog msg "Sending email notification"

  if [catch {smtp_send_email $body} result] {
    action_syslog msg "smtp_send_email: $result"
  }

  action_syslog msg "Email notification sent"
}
  
#------------------ end of send mail --------------------

  action_syslog msg "Script completed!"


#--------------------- cli close ------------------------
#
  cli_close $cli(fd) $cli(tty_id)

# eeeeeeeeeeeeeeeeeeeeeeeeeeee    End of vpn_failure.tcl eeeeeeeeeeeeeeeeeeeeee 
